import { Direction } from './types';

export default class GridDropZone {
    direction = Direction.Horizontal;
    id: string;
    priority: number;
    itemSize: number;
    count: number;
    itemsPerRow: number;
    el: HTMLDivElement;
    items: HTMLDivElement[];
    containerClass: string;

    constructor(id: string, count: number, itemsPerRow: number, priority: number, itemSize: number) {
        this.id = id;
        this.priority = priority;
        this.count = count;
        this.itemsPerRow = itemsPerRow;
        this.itemSize = itemSize;
        this.items = new Array(count);
        this.el = undefined;
        this.containerClass = 'grid';
    }

    pointIndex(x: number, y: number): number {
        const { el, itemSize, count, itemsPerRow } = this;
        const b = el.getBoundingClientRect();
        const left = b.left - el.scrollLeft;
        const top = b.top - el.scrollTop;
        const col = Math.floor((x - left) / itemSize);
        const row = Math.floor((y - top) / itemSize);
        const index = row * itemsPerRow + col;
        return Math.min(Math.max(index, 0), count);
    }

    placeholderStyleStr(): string {
        return `width: ${this.itemSize}px; height: ${this.itemSize}px;`;
    }

    dragXOffset(index: number): number {
        const { itemSize, itemsPerRow } = this;
        const col = index % itemsPerRow;
        const b = this.el.getBoundingClientRect();
        return col * itemSize + b.left - this.el.scrollLeft;
    }

    dragYOffset(index: number): number {
        const { itemSize, itemsPerRow } = this;
        const row = Math.floor(index / itemsPerRow);
        const b = this.el.getBoundingClientRect();
        return row * itemSize + b.top - this.el.scrollTop;
    }

    itemHeight(): number {
        return this.itemSize;
    }

    itemWidth(): number {
        return this.itemSize;
    }

    styleSourceMove(hover: number, source: number, transition: boolean): void {
        const { items, itemSize, itemsPerRow } = this;
        for (let i = 0; i < items.length; ++i) {
            const item = items[i];
            const base = (hover > source && (i < source || (i > source && i <= hover))) ||
                (hover < source && i < hover) ||
                (hover == source && i < source);
            const raise = (hover > source && i > hover) ||
                (hover < source && ((i >= hover && i < source) || i > source)) ||
                (hover == source && i > source);

            if (base) {
                item && (item.style.cssText = `transform: translate(0px, 0px); transition: transform 0.2s cubic-bezier(0.2, 0, 0, 1); width: ${itemSize}px; height: ${itemSize}px;`);
            } else if (raise) {
                const colOffset = (i % itemsPerRow) === (source % itemsPerRow) ? 0 : itemSize;
                const rowOffset = Math.floor(i / itemsPerRow) === Math.floor(source / itemsPerRow) ? 0 : itemSize;
                if (transition) {
                    item && (item.style.cssText = `transform: translate(${colOffset}px, ${rowOffset}px); transition: transform 0.2s cubic-bezier(0.2, 0, 0, 1); width: ${itemSize}px; height: ${itemSize}px;`);
                } else {
                    item && (item.style.cssText = `transform: translate(${colOffset}px, ${rowOffset}px); width: ${itemSize}px; height: ${itemSize}px;`);
                }
            }
        }
    }

    styleSourceMissing(index: number): void {
        const { items, itemSize } = this;
        for (let i = 0; i < items.length; ++i) {
            const item = items[i];
            item && i !== index && (items[i].style.cssText = `transform: translate(0px, 0px); transition: transform 0.2s cubic-bezier(0.2, 0, 0, 1); width: ${itemSize}px; height: ${itemSize}px;`);
        }
    }

    styleDestMove(index: number): void {
        const { items, itemSize, el, itemsPerRow } = this;
        el.style.cssText = `transition: padding-bottom 0.2s cubic-bezier(0.2, 0, 0, 1); padding-right: ${itemSize}px;`;
        for (let i = 0; i < items.length; ++i) {
            const item = items[i];
            if (i < index) {
                item && (item.style.cssText = `transform: translate(0px, 0px); transition: transform 0.2s cubic-bezier(0.2, 0, 0, 1); width: ${itemSize}px; height: ${itemSize}px;`);
            } else {
                const colOffset = (i % itemsPerRow) === (index % itemsPerRow) ? 0 : itemSize;
                const rowOffset = Math.floor(i / itemsPerRow) === Math.floor(index / itemsPerRow) ? 0 : itemSize;
                item && (item.style.cssText = `transform: translate(${colOffset}px, ${rowOffset}px); transition: transform 0.2s cubic-bezier(0.2, 0, 0, 1); width: ${itemSize}px; height: ${itemSize}px;`);
            }
        }
    }

    styleDestReset(): void {
        const { items, itemSize, el } = this;
        el.style.cssText = `transition: padding-bottom 0.2s cubic-bezier(0.2, 0, 0, 1); padding-bottom: 0px;`;
        for (let i = 0; i < items.length; ++i) {
            const item = items[i];
            item && (items[i].style.cssText = `transform: translate(0px, 0px); transition: transform 0.2s cubic-bezier(0.2, 0, 0, 1); width: ${itemSize}px; height: ${itemSize}px;`);
        }
    }

    styleRemove(): void {
        this.styleContainerBaseStyle();
    }

    styleContainerBaseStyle(): void {
        const { items, itemSize, el } = this;
        el.style.cssText = '';
        for (let i = 0; i < items.length; ++i) {
            const item = items[i];
            item && (item.style.cssText = `width: ${itemSize}px; height: ${itemSize}px;`);
        }
    }
}
